#!/usr/bin/python
# -*- encoding: latin-1 -*-

import sys, os, Image, optparse, string
import tkFileDialog         # move tk import to tk_gui() ?
import Tkinter as tk        # so Tkinter won't be loaded in command line mode

__version__        = "0.0.10"
__author__        = "Sebastian Koch (aka: Mr_Snede)"
__contact__        = "mr_snede *at* web ~dot~ de"
__homepage__    = "www ~dot~ mr-snede ~dot~ de ~dot~ vu"
__licence__        = "not yet choosen  -->gpl, python, ...?"
__requirements__= "Python >2.3 (caused by optparse) ,Tkinter"
__description__ = "Generates a thumbnail for every pic in a folder with tk GUI or full commandline usage"

"""
    Copyright:      2005 by Sebastian Koch (headstones_at_web_dot_ de)
    Last modified:  2006-06-28
    minor version: return_string
        changes in this minor version:
        change create_thumb()'s return argument from tuple to preformated printable string
        create_thumb() is called in main [line 267]
        create_thumb() is called in create_them [line 168]
"""

# Comment usage:
# <idea>    notes, tipps, thoughts for future changes
# <fixme>
# #----       general info for following statement(s)

#----------------------------------------------------------
def parse_options():
    """parses and returns command line options"""
    # ---- define usage-text
    usage = "\n    %prog [thumb_height] \n\n" + \
            "Description: " + "\n    " + __description__ + "\n\nAuthor:\n    " + __author__
    version = "%prog " + __version__

    # ---- create parser object
    parser = optparse.OptionParser(usage = usage, version = version)

    # ---- set command line options and their defaults
    parser.add_option("-y", "--thumb_height",
        action="store", type="int", dest="thumb_height", default = 500,
        help = "(in pixel) used to calculate thumb_width by aspectratio of inputpic --> default = 100")
    parser.add_option("-p", "--pic_list",
        action = "store", type = "string", dest = "pic_list", default = "jpg,jpeg,jp2,png,gif,bmp,tiff",
        help = "Image formats that will be processed --> default = jpg, jpeg, jp2, png, gif, bmp, tiff")
    parser.add_option("-d", "--directory",
        action="store", type="string", dest="pic_dir", default = os.getcwd(),
        help = "Path where the pictures lay to be 'thumbnaild' --> default = current working directory")
    parser.add_option("-g", "--gui",
        action="store", type="string", dest="gui", default = "tk",
        help = "Choose gui: 'text' for noninteractive command line interface or 'tk' for a tkinter window  --> default = tk")

    # ---- parse options
    (options, args) = parser.parse_args()
    # ---- convert string of pic types "jpg, png, ..." ---> list of pic types ['jpg', 'png', ...]
    options.pic_list = options.pic_list.split(',')

    return (options, args)


#----------------------------------------------------------
def start_info():
    """returns a general usageinfo and default settings"""
    if len(sys.argv) <=1: # <idea> actualy this case wont be reached in any case (caused by defaults?)
        info_text = """Running without Arguments by using following defaults:
-->Thumb_heigt = 100,  pic_list = jpg, jpeg, jp2, png, gif, bmp, tiff
--> pic_dir = %s\nFor help run: %s -h or --help""" %(os.getcwd(), sys.argv[0])
    else:
        info_text = "For help run: %s -h or --help \n\n" %(sys.argv[0])

    return info_text


#----------------------------------------------------------
def create_thumb(file_full_path, thumb_height):
    """ creates a thumb of a pic by a given height with original aspectratio
        Paramters:
            file_full_path = path/to/file.ext
            thumb_height =  (in pixel) used to calculate thumb_width by aspectratio of inputpic
    """
    no_extension = os.path.splitext(file_full_path)[0]

    im = Image.open(file_full_path)
    original_width, original_height = im.size
    if original_height > thumb_height:
        thumb_width = thumb_height * original_width / original_height
        try:
            im.thumbnail((thumb_width, thumb_height), Image.ANTIALIAS)
            im.save(no_extension + "-thumb.jpg", "JPEG")
            pic_info = (original_height, original_width, thumb_height, thumb_width, os.path.basename(no_extension) )
            return """\n :-)  -- Thumb created:
height x width: [Original: %dx%d] - [Thumb: %dx%d] --> %s.jpg """ %pic_info
        except ZeroDivisionError:
            pic_info = (original_height, original_width, os.path.basename(no_extension) )
            return """\n :-(  -- Passing!! Original is to thin:
height x width: [Original: %dx%d] --> %s.jpg """ %pic_info

    else:
        pic_info = (original_height, original_width, os.path.basename(no_extension) )
        return """\n :-(  -- Passing!! pic is smaller than your chosen height:
height x width: [Original: %dx%d] --> %s.jpg """ %pic_info

#----------------------------------------------------------
def create_pic_list(input_folder, pic_list):
    """ creates a List of all pics in "inputfolder", pictype is defined by pic_list
        Paramters:
            inputfolder = folder with original pics
            pic_list = list of filetype endings to define files as pictures (ie jpg or png)
            !! ------ rename pic_list to pictype_list -------!!
    """
    picture_list = []
    nopicture_list = []

    for this_file in os.listdir(input_folder):
        if os.path.isfile(os.path.join(input_folder, this_file)) == 1:
            if string.lower(os.path.splitext(this_file)[1][1:]) in pic_list:
                picture_list.append(os.path.join(input_folder, this_file))
            else:
                nopicture_list.append(this_file)
        else:
            nopicture_list.append(os.path.join(input_folder, this_file))

    return [picture_list, nopicture_list]


#----------------------------------------------------------
def tk_gui(pic_dir, thumb_height, pic_list):
    """creates a Tkinter GUI"""

    #----------------------------------------------------------
    def print_help(event=None):
        """prints programm infos and  help into txt_output"""
        txt_output.delete("1.0", tk.END)
        txt_output.insert(tk.END, " -- Help/Programm Info -- \n\n")
        txt_output.insert(tk.END, start_info())
        # <idea> this Info has to be extendet: usage, author, dependences, licence...

    #----------------------------------------------------------
    def open_it(event=None): # <idea> change to pic_dir (*_pic_folder)?
        """chooses a directory with imagefiles to convert"""

        path_temp = tkFileDialog.askdirectory()
        try :
            btn_pic_dir["text"] = path_temp
            print_filelist(path_temp)
        except:
            btn_pic_dir["text"] = pic_dir
            print_filelist(pic_dir)

    #----------------------------------------------------------
    def print_filelist(pic_dir):
        """prints all image and no_imagefiles into txt_output"""
        txt_output.delete("1.0", tk.END)
        txt_output.insert(tk.END, " -- Press F1 for Help/Programm Info -- \n ")
        txt_output.tag_add("info_color", "1.0", "1.end")
        txt_output.tag_configure("info_color", foreground="#007500", background="grey", justify=tk.CENTER)
        txt_output.insert(tk.END, "\n\nThumbs can be generated from following Pictures:\n ")
        for i in create_pic_list(pic_dir, pic_list)[0]:
            txt_output.insert(tk.END, "\n%s" %os.path.basename(i))

        txt_output.insert(tk.END, "\n\nIgnoring following files/folders:\n ")
        for i in create_pic_list(pic_dir, pic_list)[1]:
            txt_output.insert(tk.END, "\n%s" %os.path.basename(i))

        txt_output.see(tk.END)

    #----------------------------------------------------------
    def create_them(event=None): # <info> move into core?
        """calls create_thumbs, writes results into txt_output"""
        for i in create_pic_list(btn_pic_dir.cget("text"),pic_list)[0]:
            output_string = create_thumb(i, int(entry_thumb_height.get()) )
            txt_output.insert(tk.END, output_string)
            txt_output.see(tk.END)
            txt_output.update()

    #----------------------------------------------------------
    def focus_entry_thumb_height(event=None):
        """sets focus to entry_thumb_height and marks all text inside"""
        entry_thumb_height.focus_set()
        entry_thumb_height.selection_range(0,"end")

    master = tk.Tk()

    # ---- widget creation
    lbl_pic_dir = tk.Label(master, text = "Picture Directory:")
    btn_pic_dir = tk.Button(master, text = "Picture Directory")
    lbl_thumb_height = tk.Label(master, text = "Thumb Height:")
    entry_thumb_height = tk.Entry(master)
    btn_ende = tk.Button(master, text = "Quit")
    btn_go = tk.Button(master, text = "Go")

    frame_output = tk.Frame(master, relief=tk.RIDGE, borderwidth = 2)
    txt_output=tk.Text(frame_output)
    verticalScroller = tk.Scrollbar(frame_output, orient="vertical")
    horizontalScroller = tk.Scrollbar(frame_output, orient="horizontal")


    # ---- basic placement (the sequence defines the activation order by "TAB" keystroke)
    lbl_pic_dir.grid(row = 0, column = 0, sticky = "nw")
    btn_pic_dir.grid(row = 0, column = 1, sticky = "enw")
    lbl_thumb_height.grid(row = 1, column = 0, sticky = "w")
    entry_thumb_height.grid(row=1, column=1, sticky="ew")
    btn_go.grid(row=2, column=1, sticky="e")
    btn_ende.grid(row=2, column=0, sticky="w")

    frame_output.grid(row = 3, columnspan = 2, sticky="nesw")
    txt_output.grid(row=0, column=0, sticky="nesw")
    verticalScroller.grid(row=0, column=1, sticky="ns")
    horizontalScroller.grid(row=1, column=0, sticky="ew")

    # ---- resieze settings (yet not optimal)
    master.resizable(1,0)
    master.columnconfigure(1, weight=1)
    master.rowconfigure(1, weight=1)
    frame_output.columnconfigure(0, weight=1)
    frame_output.rowconfigure(0, weight=1)

    # ---- enhanced widget settings (style + funktions)
    master.title("tk-tk")
    btn_pic_dir.config(relief = tk.SUNKEN, text = pic_dir, command = open_it, bg = "white")
    lbl_pic_dir.config(underline = 0)
    lbl_thumb_height.config(underline=0)
    entry_thumb_height.config(bg = "white")
    entry_thumb_height.insert(0, thumb_height)
    btn_ende.config(relief=tk.GROOVE, fg="#B00000", command=master.destroy, underline=0) #B00000 = red
    btn_go.config(relief = tk.GROOVE, fg = "#007500", underline = 0)#007500 = green
    btn_go.config(command = create_them)
    txt_output.config(takefocus=0)

    # ---- combining scrollbars with textwidget
    txt_output['yscrollcommand'] = verticalScroller.set
    txt_output['xscrollcommand'] = horizontalScroller.set
    verticalScroller['command'] = txt_output.yview
    horizontalScroller['command'] = txt_output.xview

    # ---- Keybindings
    master.bind_all("<Escape>", lambda event: master.destroy() )
    master.bind_all("<Control-q>", lambda event: master.destroy() )
    master.bind_all("<F1>", print_help)
    master.bind_all("<Control-h>", print_help)
    master.bind_all("<F2>", open_it)
    master.bind_all("<Control-o>", open_it)
    master.bind_all("<F3>", focus_entry_thumb_height)
    master.bind_all("<Control-y>", focus_entry_thumb_height)
    master.bind_all("<F4>", create_them)
    master.bind_all("<Control-g>", create_them)

    print_filelist(pic_dir)

    master.mainloop()


#----------------------------------------------------------

def main():
    # ---- parse options and arguments
    (options, args) = parse_options()

    # ---- switch between available GUI's
    if options.gui == "tk":
        tk_gui(options.pic_dir, options.thumb_height, options.pic_list)
    else:         # <idea> elif when there are more guis to come
        print "%s" %start_info()
        for i in create_pic_list(options.pic_dir, options.pic_list)[0]:
            pic_info = create_thumb(i, options.thumb_height)
            print pic_info


if __name__ == '__main__':
    main()
